import tkinter as tk
from tkinter import filedialog, scrolledtext, messagebox
import tkinter.ttk as ttk
import zipfile
import subprocess
import re
import os
import shutil
import threading
import sys
import traceback

# --- הגדרות (ללא שינוי מהותי) ---
PREFERRED_DENSITIES_ORDER = [
    'xxxhdpi', 'xxhdpi', 'xhdpi', 'hdpi', 'mdpi', 'ldpi', 'anydpi'
]
COMMON_ICON_NAMES = [ # הרחבתי מעט יותר עם רכיבי adaptive
    'ic_launcher.png', 'icon.png', 'ic_launcher_round.png', 'app_icon.png',
    'logo.png', 'appicon.png', 'launcher_icon.png',
    'ic_launcher_foreground.png', 'ic_launcher_background.png',
    'ic_launcher_foreground.webp', 'ic_launcher_background.webp'
]
# חשוב להוסיף .xml כשנחפש אותו באופן ספציפי
ADAPTIVE_ICON_XML_NAMES = ['ic_launcher.xml', 'icon.xml', 'app_icon.xml', 'launcher_icon.xml']

IMAGE_EXTENSIONS = ('.png', '.webp', '.jpg', '.jpeg', '.gif', '.ico')

# --- פונקציות עזר (רוב הפונקציות ללא שינוי או עם שינויים מינוריים בהודעות) ---

def log_message(text_area, message):
    if text_area and hasattr(text_area, 'winfo_exists') and text_area.winfo_exists():
        current_state = text_area.cget("state")
        text_area.configure(state='normal')
        text_area.insert(tk.END, message + "\n")
        text_area.see(tk.END)
        if current_state == 'disabled':
             text_area.configure(state='disabled')
        text_area.update_idletasks()
    else:
        print(f"LOG (fallback): {message}", file=sys.stderr)


def find_best_icon_path_from_list(icon_paths, log_area, prefer_png_over_webp=True, specific_filename=None):
    best_icon_candidate = None
    best_density_index = float('inf')
    best_is_png = False

    image_icon_paths = [
        p for p in icon_paths 
        if p.lower().endswith(IMAGE_EXTENSIONS) and not p.lower().endswith('.xml')
    ]
    
    if specific_filename: # אם מחפשים שם קובץ ספציפי (למשל, רכיב של אייקון אדפטיבי)
        image_icon_paths = [p for p in image_icon_paths if os.path.basename(p.lower()) == specific_filename.lower()]
        if not image_icon_paths:
            log_message(log_area, f"    בחירת נתיב: לא נמצא הקובץ הספציפי '{specific_filename}' ברשימת המועמדים לתמונות.")
            return None


    if not image_icon_paths:
        xml_icon_paths = [p for p in icon_paths if p.lower().endswith('.xml')]
        if xml_icon_paths and not specific_filename : # הודעה רק אם לא מחפשים קובץ ספציפי
            log_message(log_area, f"    בחירת נתיב: נמצאו רק אייקונים מסוג XML: {xml_icon_paths[:3]}...")
        elif not xml_icon_paths and not specific_filename:
            log_message(log_area, "    בחירת נתיב: לא נמצאו נתיבי קבצי תמונה או XML ברשימת המועמדים.")
        return None

    for path in image_icon_paths:
        path_lower = path.lower()
        current_density_index = float('inf')
        found_density_in_path = False
        
        for i, density in enumerate(PREFERRED_DENSITIES_ORDER):
            if f'-{density}' in path_lower or f'/{density}/' in path_lower or \
               f'drawable-{density}' in path_lower or f'mipmap-{density}' in path_lower :
                current_density_index = i
                found_density_in_path = True
                break
        
        if not found_density_in_path:
            current_density_index = len(PREFERRED_DENSITIES_ORDER) + 1 
            # log_message(log_area, f"    בחירת נתיב: לא זוהתה רזולוציה ברורה עבור '{path}', מקבל עדיפות נמוכה.")
        
        is_png = path_lower.endswith('.png')

        if current_density_index < best_density_index:
            best_density_index = current_density_index
            best_icon_candidate = path
            best_is_png = is_png
        elif current_density_index == best_density_index:
            if prefer_png_over_webp and is_png and not best_is_png:
                best_icon_candidate = path
                best_is_png = True
            elif not prefer_png_over_webp and path_lower.endswith('.webp') and not best_icon_candidate.lower().endswith('.webp'):
                best_icon_candidate = path # אם מעדיפים WebP
            elif 'ic_launcher.png' in path_lower and 'ic_launcher_round.png' in (best_icon_candidate.lower() if best_icon_candidate else ""):
                best_icon_candidate = path 
                best_is_png = is_png
    
    if best_icon_candidate and not specific_filename: # הדפס הודעה רק אם לא מחפשים רכיב ספציפי (פחות "רעש" בלוג)
        log_message(log_area, f"    בחירת נתיב: המועמד שנבחר הוא '{best_icon_candidate}' (איכות DPI {best_density_index}).")
    elif not best_icon_candidate and not specific_filename:
        log_message(log_area, "    בחירת נתיב: לא נבחר אייקון תמונה מבין המועמדים.")
        
    return best_icon_candidate


def get_aapt_path(log_area_for_startup=None):
    # ... (קוד הפונקציה נשאר זהה לגרסה הקודמת) ...
    aapt_exe_name = "aapt.exe" if os.name == 'nt' else "aapt"
    aapt_paths_to_try = [aapt_exe_name]
    sdk_env_vars = ['ANDROID_HOME', 'ANDROID_SDK_ROOT']
    sdk_roots = [os.environ.get(var) for var in sdk_env_vars if os.environ.get(var)]
    common_sdk_paths_windows = [os.path.join(os.environ.get('LOCALAPPDATA', ''), 'Android', 'Sdk'), 'C:\\Android\\sdk']
    common_sdk_paths_linux_mac = [os.path.join(os.environ.get('HOME', ''), 'Android/Sdk'), os.path.join(os.environ.get('HOME', ''), 'android-sdk'), '/usr/lib/android-sdk', '/opt/android-sdk']
    if os.name == 'nt': sdk_roots.extend(common_sdk_paths_windows)
    else: sdk_roots.extend(common_sdk_paths_linux_mac)
    for sdk_root in sdk_roots:
        if sdk_root and os.path.isdir(sdk_root):
            build_tools_dir = os.path.join(sdk_root, "build-tools")
            if os.path.isdir(build_tools_dir):
                versions = sorted([d for d in os.listdir(build_tools_dir) if os.path.isdir(os.path.join(build_tools_dir, d)) and d.replace('.', '').isdigit()], key=lambda v: list(map(int, v.split('.'))), reverse=True)
                if versions:
                    potential_aapt_path = os.path.join(build_tools_dir, versions[0], aapt_exe_name)
                    if potential_aapt_path not in aapt_paths_to_try: aapt_paths_to_try.append(potential_aapt_path)
    for path_try in aapt_paths_to_try:
        try:
            resolved_path = shutil.which(path_try)
            if resolved_path and os.access(resolved_path, os.X_OK):
                startupinfo = None
                if os.name == 'nt':
                    startupinfo = subprocess.STARTUPINFO()
                    startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
                    startupinfo.wShowWindow = subprocess.SW_HIDE
                process = subprocess.run([resolved_path, "version"], capture_output=True, text=True, startupinfo=startupinfo, timeout=3)
                if process.returncode == 0 and "Android Asset Packaging Tool" in process.stdout:
                    if log_area_for_startup: log_message(log_area_for_startup, f"  בדיקת aapt: נמצא בנתיב מאומת: {resolved_path}")
                    return resolved_path
        except: continue
    if log_area_for_startup: log_message(log_area_for_startup, "  בדיקת aapt: הפקודה 'aapt' לא אותרה.")
    return None

# --- שינויים עיקריים ב-extract_icon_using_aapt ---
def extract_icon_using_aapt(apk_path, output_dir, log_area):
    aapt_cmd = get_aapt_path()
    if not aapt_cmd:
        log_message(log_area, "  חילוץ aapt: לא ניתן למצוא 'aapt'.")
        return False, None # צריך להחזיר False אם נכשל

    log_message(log_area, f"  חילוץ aapt: משתמש ב-{aapt_cmd} עבור {os.path.basename(apk_path)}...")
    try:
        startupinfo = None
        if os.name == 'nt':
            startupinfo = subprocess.STARTUPINFO()
            startupinfo.dwFlags |= subprocess.STARTF_USESHOWWINDOW
            startupinfo.wShowWindow = subprocess.SW_HIDE
        
        process = subprocess.run([aapt_cmd, 'dump', 'badging', apk_path], 
                                 capture_output=True, text=True, encoding='utf-8', errors='ignore',
                                 startupinfo=startupinfo)

        if process.returncode != 0:
            log_message(log_area, f"  שגיאה: aapt נכשל (קוד {process.returncode}). stderr: {process.stderr.strip()[:200]}...")
            return False, None

        badging_output = process.stdout
        apk_basename_no_ext = os.path.splitext(os.path.basename(apk_path))[0]
        
        # 1. חפש אייקון תמונה ישיר
        aapt_direct_image_icons = []
        direct_matches = re.findall(r"application-icon(?:-\d+)?:\s*'([^']+)'", badging_output)
        aapt_direct_image_icons.extend(direct_matches)
        main_icon_match = re.search(r"application(?:-\w+)*:\s*label='(?:[^']*)'\s*icon='([^']+)'", badging_output)
        if main_icon_match:
            declared_main_icon = main_icon_match.group(1)
            if declared_main_icon not in aapt_direct_image_icons:
                aapt_direct_image_icons.append(declared_main_icon)
        
        # סנן רק נתיבי תמונות מהדיווח הישיר
        aapt_direct_image_icons_filtered = [p for p in aapt_direct_image_icons if p.lower().endswith(IMAGE_EXTENSIONS)]

        if aapt_direct_image_icons_filtered:
            log_message(log_area, f"    aapt דיווח על אייקוני תמונה ישירים: {aapt_direct_image_icons_filtered}")
            icon_to_extract = find_best_icon_path_from_list(aapt_direct_image_icons_filtered, log_area)
            if icon_to_extract:
                log_message(log_area, f"    חילוץ אייקון תמונה ישיר: {icon_to_extract}")
                # ... (תהליך החילוץ הישיר כפי שהיה)
                with zipfile.ZipFile(apk_path, 'r') as zip_ref:
                    if icon_to_extract not in zip_ref.namelist(): return False, None
                    icon_original_ext = os.path.splitext(icon_to_extract)[1].lower()
                    icon_filename_ext = icon_original_ext if icon_original_ext in IMAGE_EXTENSIONS else ".png"
                    output_icon_name = f"{apk_basename_no_ext}_icon_aapt{icon_filename_ext}"
                    # ... (לוגיקת מניעת דריסה)
                    counter = 1; base, ext = os.path.splitext(output_icon_name)
                    output_icon_path_final = os.path.join(output_dir, output_icon_name)
                    while os.path.exists(output_icon_path_final):
                        output_icon_name = f"{base}_{counter}{ext}"; output_icon_path_final = os.path.join(output_dir, output_icon_name); counter += 1
                    with open(output_icon_path_final, 'wb') as f_out: f_out.write(zip_ref.read(icon_to_extract))
                    log_message(log_area, f"  הצלחה (aapt-ישיר): חולץ אל {output_icon_path_final}")
                    return True, output_icon_path_final
            else:
                 log_message(log_area, "    aapt דיווח על אייקוני תמונה אך לא נבחר מועמד איכותי (או כולם לא תמונות).")


        # 2. אם לא נמצא אייקון תמונה ישיר, חפש Adaptive Icon
        log_message(log_area, "    לא נמצא אייקון תמונה ישיר דרך aapt. בודק Adaptive Icon...")
        adaptive_icon_xml_path = None
        adaptive_match = re.search(r"application(?:-\w+)*:\s*adaptive-icon='([^']+)'", badging_output)
        if adaptive_match:
            adaptive_icon_xml_path = adaptive_match.group(1)
            log_message(log_area, f"    aapt דיווח על adaptive-icon XML: '{adaptive_icon_xml_path}'")

            # נסה לחלץ רכיבי foreground/background אם ה-XML מצביע על קונבנציות ידועות
            # זוהי הנחה והיא עשויה לא לעבוד תמיד, אבל שווה ניסיון
            if adaptive_icon_xml_path.lower().endswith(".xml"):
                xml_base_name = os.path.splitext(os.path.basename(adaptive_icon_xml_path))[0] # למשל 'ic_launcher'
                
                components_extracted_count = 0
                with zipfile.ZipFile(apk_path, 'r') as zip_ref:
                    all_files_in_apk = zip_ref.namelist()
                    
                    for component_type in ["foreground", "background"]:
                        # נסה גם PNG וגם WebP
                        component_filenames_to_try = [
                            f"{xml_base_name}_{component_type}.png",
                            f"{xml_base_name}_{component_type}.webp"
                        ]
                        
                        # חפש את רכיב ה-adaptive בתיקיות mipmap השונות
                        candidate_component_paths = []
                        for item_in_zip in all_files_in_apk:
                            if item_in_zip.lower().startswith("res/mipmap") and \
                               os.path.basename(item_in_zip.lower()) in [fn.lower() for fn in component_filenames_to_try]:
                                candidate_component_paths.append(item_in_zip)
                        
                        if candidate_component_paths:
                            log_message(log_area, f"    מועמדים לרכיב Adaptive '{component_type}': {candidate_component_paths}")
                            # מצא את הרכיב הטוב ביותר (DPI הגבוה ביותר)
                            best_component_path = find_best_icon_path_from_list(
                                candidate_component_paths, log_area, 
                                specific_filename=os.path.basename(candidate_component_paths[0]) # רק לשם סינון ראשוני לפי השם המלא
                            )
                            # אם הקודם נכשל (למקרה שהרשימה לא הייתה ממוינת), נסה שוב בלי specific_filename על הרשימה המקורית
                            if not best_component_path and len(candidate_component_paths)>1 :
                                 best_component_path = find_best_icon_path_from_list(candidate_component_paths, log_area)


                            if best_component_path:
                                log_message(log_area, f"    נבחר רכיב Adaptive '{component_type}': '{best_component_path}'")
                                component_ext = os.path.splitext(best_component_path)[1]
                                output_component_name = f"{apk_basename_no_ext}_adaptive_{component_type}{component_ext}"
                                # ... (לוגיקת מניעת דריסה)
                                counter = 1; base_c, ext_c = os.path.splitext(output_component_name)
                                output_comp_path_final = os.path.join(output_dir, output_component_name)
                                while os.path.exists(output_comp_path_final):
                                    output_component_name = f"{base_c}_{counter}{ext_c}"; output_comp_path_final = os.path.join(output_dir, output_component_name); counter += 1
                                
                                with open(output_comp_path_final, 'wb') as f_out:
                                    f_out.write(zip_ref.read(best_component_path))
                                log_message(log_area, f"  הצלחה (aapt-adaptive): רכיב '{component_type}' חולץ אל {output_comp_path_final}")
                                components_extracted_count += 1
                            else:
                                log_message(log_area, f"    לא נבחר קובץ תמונה תקף לרכיב Adaptive '{component_type}'.")
                        else:
                             log_message(log_area, f"    לא נמצאו מועמדים לרכיב Adaptive '{component_type}' עם השמות: {component_filenames_to_try}")


                if components_extracted_count > 0:
                    return True, "Adaptive components extracted" # לא נתיב יחיד, אלא הצלחה כללית
                else:
                    log_message(log_area, "    חילוץ aapt (adaptive): לא הצליח לחלץ רכיבי foreground/background כקבצי תמונה.")
                    return False, None # עדיין נכשל אם לא חולצו רכיבים

        log_message(log_area, "  חילוץ aapt: לא נמצא אייקון (ישיר או אדפטיבי-תמונה) דרך aapt.")
        return False, None # כישלון סופי של שיטת aapt

    except FileNotFoundError:
        log_message(log_area, "  שגיאה (aapt): הפקודה 'aapt' לא נמצאה למרות הניסיון לאתר אותה.")
        return False, None
    except Exception as e:
        log_message(log_area, f"  שגיאה לא צפויה בשימוש ב-aapt: {type(e).__name__} - {e}")
        # פרטי שגיאה מלאים עלולים להיות ארוכים מדי, רק סוג וכותרת.
        # full_trace = traceback.format_exc()
        # log_message(log_area, f"  פרטי שגיאה (aapt): {full_trace.splitlines()[-1]}")
        return False, None

# --- פונקציית extract_icon_manually וכל שאר הקוד נשארים כמעט זהים ---
# ... (כאן יבוא שאר הקוד ללא שינויים משמעותיים מהגרסה הקודמת)

def extract_icon_manually(apk_path, output_dir, log_area):
    # ... (קוד זהה לגרסה V3 שהבאת קודם, למעט אולי הדפסות לוג מפורטות יותר אם רוצים)
    log_message(log_area, f"  חילוץ ידני: עבור {os.path.basename(apk_path)}...")
    try:
        with zipfile.ZipFile(apk_path, 'r') as zip_ref:
            namelist = zip_ref.namelist()
            candidate_icon_paths = []
            search_log = []

            for item_path in namelist:
                item_path_lower = item_path.lower()
                if (item_path_lower.startswith('res/drawable-') or item_path_lower.startswith('res/mipmap-')) and \
                   any(name in item_path_lower for name in COMMON_ICON_NAMES) and \
                   item_path_lower.endswith(IMAGE_EXTENSIONS) and not item_path_lower.endswith('.xml'):
                    candidate_icon_paths.append(item_path)
            search_log.append(f"מצא ב-res/*-dpi: {len(candidate_icon_paths)} מועמדים.")

            if not candidate_icon_paths:
                for item_path in namelist:
                    item_path_lower = item_path.lower()
                    if (item_path_lower.startswith('res/drawable/') or item_path_lower.startswith('res/mipmap/')) and \
                       not (item_path_lower.startswith('res/drawable-') or item_path_lower.startswith('res/mipmap-')) and \
                       any(name in item_path_lower for name in COMMON_ICON_NAMES) and \
                       item_path_lower.endswith(IMAGE_EXTENSIONS) and not item_path_lower.endswith('.xml'):
                        candidate_icon_paths.append(item_path)
                search_log.append(f"לאחר res/ (בלי DPI): {len(candidate_icon_paths)} מועמדים.")
            
            if not candidate_icon_paths:
                for item_path in namelist:
                    item_path_lower = item_path.lower()
                    if '/' not in item_path_lower.replace("res/", ""):
                         if any(name == os.path.basename(item_path_lower) for name in COMMON_ICON_NAMES) and \
                            item_path_lower.endswith(IMAGE_EXTENSIONS) and not item_path_lower.endswith('.xml'):
                             candidate_icon_paths.append(item_path)
                search_log.append(f"לאחר חיפוש גלובלי: {len(candidate_icon_paths)} מועמדים.")

            log_message(log_area, f"    חילוץ ידני: סטטוס חיפוש: {'; '.join(search_log)}")
            if not candidate_icon_paths:
                log_message(log_area, f"  חילוץ ידני: לא נמצאו קבצי אייקונים תמונה פוטנציאליים כלל.")
                return False, None

            candidate_icon_paths = sorted(list(set(candidate_icon_paths)))
            log_message(log_area, f"    חילוץ ידני: סה\"כ מועמדים ייחודיים: {candidate_icon_paths if len(candidate_icon_paths) < 6 else str(candidate_icon_paths[:5])+'...'})")
            icon_to_extract = find_best_icon_path_from_list(candidate_icon_paths, log_area)
            if not icon_to_extract:
                log_message(log_area, f"  חילוץ ידני: לא נבחר אייקון תמונה (ייתכן שכולם סוננו או XML).")
                return False, None

            log_message(log_area, f"    חילוץ ידני: האייקון שנבחר: {icon_to_extract}")
            apk_basename = os.path.splitext(os.path.basename(apk_path))[0]
            icon_filename_ext = os.path.splitext(icon_to_extract)[1]
            if not icon_filename_ext: icon_filename_ext = ".png"
            output_icon_name = f"{apk_basename}_icon_manual{icon_filename_ext}"
            output_icon_path = os.path.join(output_dir, output_icon_name)
            counter = 1; base, ext = os.path.splitext(output_icon_name)
            while os.path.exists(output_icon_path):
                output_icon_name = f"{base}_{counter}{ext}"; output_icon_path = os.path.join(output_dir, output_icon_name); counter += 1
            with open(output_icon_path, 'wb') as f_out: f_out.write(zip_ref.read(icon_to_extract))
            log_message(log_area, f"  הצלחה (ידני): האייקון חולץ אל {output_icon_path}")
            return True, output_icon_path
    except zipfile.BadZipFile:
        log_message(log_area, f"  שגיאה (ידני): {os.path.basename(apk_path)} אינו APK תקין.")
        return False, None
    except Exception as e:
        log_message(log_area, f"  שגיאה לא צפויה בחילוץ ידני: {type(e).__name__} - {e}")
        return False, None

def process_apk_files(apk_files, output_dir, log_area, progress_callback):
    # ... (כמו קודם)
    if not apk_files: log_message(log_area, "לא נבחרו קבצי APK."); progress_callback(True); return
    if not output_dir: log_message(log_area, "לא נבחרה תיקיית פלט."); progress_callback(True); return
    try: os.makedirs(output_dir, exist_ok=True)
    except OSError as e: log_message(log_area, f"שגיאה ביצירת תיקיית פלט: {e}"); progress_callback(True); return
    total_files = len(apk_files); extracted_count = 0
    log_message(log_area, "--- בודק זמינות aapt גלובלית פעם אחת ---")
    aapt_available_globally = get_aapt_path(log_area) is not None
    if aapt_available_globally: log_message(log_area, "aapt זוהה. תינתן לו עדיפות בחילוץ.")
    else: log_message(log_area, "aapt לא זוהה. תתבצע הסתמכות על חילוץ ידני.")
    log_message(log_area, "-----------------------------------------")
    for i, apk_path in enumerate(apk_files):
        log_message(log_area, f"\nמעבד קובץ ({i+1}/{total_files}): {os.path.basename(apk_path)}...")
        extracted_this_file = False
        if aapt_available_globally:
            success_aapt, aapt_result = extract_icon_using_aapt(apk_path, output_dir, log_area) # שינוי כאן לקבלת aapt_result
            if success_aapt:
                extracted_count += 1
                extracted_this_file = True
        if not extracted_this_file:
            if aapt_available_globally : log_message(log_area, f"  ניסיון עם aapt נכשל. עובר לשיטה הידנית...")
            else: log_message(log_area, f"  aapt לא זמין, עובר ישירות לשיטה הידנית...")
            success_manual, _ = extract_icon_manually(apk_path, output_dir, log_area)
            if success_manual: extracted_count += 1
            else: log_message(log_area, f"  חילוץ האייקון מ-{os.path.basename(apk_path)} נכשל גם בשיטה הידנית.")
    log_message(log_area, f"\n--- סיכום ---") # ... (סיכום ופתיחת תיקייה כמו קודם)
    log_message(log_area, f"סה\"כ קבצים לעיבוד: {total_files}")
    log_message(log_area, f"אייקונים (או רכיביהם) שחולצו בהצלחה: {extracted_count}")
    log_message(log_area, f"אייקונים שלא חולצו: {total_files - extracted_count}")
    if total_files - extracted_count > 0:
        log_message(log_area, "עבור קבצים שלא חולצו, אנא בדוק את הלוג המפורט לעיל.")
        log_message(log_area, "אם הבעיה נמשכת, ייתכן שהאייקון הוא וקטורי לחלוטין או בפורמט לא נתמך.")
    if extracted_count > 0:
        try:
            abs_output_dir = os.path.abspath(output_dir)
            if sys.platform == "win32": os.startfile(abs_output_dir)
            elif sys.platform == "darwin": subprocess.call(["open", abs_output_dir])
            else: subprocess.call(["xdg-open", abs_output_dir])
            log_message(log_area, f"האייקונים נשמרו בתיקייה: {abs_output_dir}")
        except Exception as e: log_message(log_area, f"האייקונים נשמרו בתיקייה: {abs_output_dir} (שגיאה בפתיחה: {e})")
    elif total_files > 0 : log_message(log_area, "לא חולצו אייקונים בתהליך זה.")
    if progress_callback: progress_callback(True)


class App:
    # ... (__init__ והגדרות GUI כמו קודם)
    def __init__(self, root):
        self.root = root
        self.root.title("מחלץ אייקונים מ-APK (גרסה V4 - Adaptive)")
        self.root.geometry("800x600")
        self.apk_files = []
        self.output_dir = ""
        self.log_area = None 
        self.default_font_family = "Segoe UI" if os.name == 'nt' else "Helvetica"
        self.default_font = (self.default_font_family, 10)
        self.log_font = ("Consolas", 9) if os.name == 'nt' else ("Monospace", 9)
        self.label_font = (self.default_font_family, 9)
        self.button_font = (self.default_font_family, 10, "bold")
        self.style = ttk.Style()
        self.style.configure('.', font=self.default_font)
        self.style.configure('TButton', font=self.button_font)
        self.style.configure('Accent.TButton', font=self.button_font, padding=(10, 5))
        main_frame = ttk.Frame(root, padding="15 15 15 15"); main_frame.pack(fill=tk.BOTH, expand=True)
        controls_frame = ttk.Frame(main_frame); controls_frame.pack(fill=tk.X, pady=(0, 15)); controls_frame.columnconfigure(1, weight=1) 
        btn_select_apks = ttk.Button(controls_frame, text="בחר קבצי APK...", command=self.select_apks, width=22); btn_select_apks.grid(row=0, column=0, padx=(0,10), pady=(0,5), sticky="ew")
        self.lbl_apk_count_var = tk.StringVar(value="לא נבחרו קבצים"); lbl_apk_count = ttk.Label(controls_frame, textvariable=self.lbl_apk_count_var, anchor="w", font=self.label_font); lbl_apk_count.grid(row=0, column=1, padx=(0,5), pady=(0,5), sticky="ew")
        btn_select_output = ttk.Button(controls_frame, text="בחר תיקיית פלט...", command=self.select_output_dir, width=22); btn_select_output.grid(row=1, column=0, padx=(0,10), pady=(5,0), sticky="ew")
        self.lbl_output_dir_var = tk.StringVar(value="לא נבחרה תיקייה"); lbl_output_dir = ttk.Label(controls_frame, textvariable=self.lbl_output_dir_var, anchor="w", font=self.label_font); lbl_output_dir.grid(row=1, column=1, padx=(0,5), pady=(5,0), sticky="ew")
        self.btn_extract = ttk.Button(main_frame, text="חלץ אייקונים", command=self.start_extraction_thread, state=tk.DISABLED, style='Accent.TButton'); self.btn_extract.pack(pady=(5, 15), ipadx=10, ipady=5)
        log_container_frame = ttk.Frame(main_frame); log_container_frame.pack(fill=tk.BOTH, expand=True); log_container_frame.configure(borderwidth=1, relief="groove")
        self.log_area = scrolledtext.ScrolledText(log_container_frame, wrap=tk.WORD, height=15, width=90); self.log_area.pack(pady=5, padx=5, fill=tk.BOTH, expand=True); self.log_area.configure(font=self.log_font, relief=tk.FLAT, bd=0, state='disabled')
        self.setup_styles_and_theme()
        log_message(self.log_area, "ברוכים הבאים! אנא בחרו קבצי APK ותיקיית פלט.")
        self.check_aapt_availability_on_startup()
    
    # ... (setup_styles_and_theme, check_aapt_availability_on_startup ופונקציות ה-GUI האחרות כמו קודם)
    def setup_styles_and_theme(self):
        try:
            log_message(self.log_area, "  מנסה להגדיר Theme ל-ttk...")
            available_themes = self.style.theme_names(); current_theme_to_set = self.style.theme_use(); preferred_themes = []
            if os.name == 'nt': preferred_themes = ['vista', 'xpnative', 'winnative', 'clam']
            elif sys.platform == 'darwin': preferred_themes = ['aqua', 'clam']
            else: preferred_themes = ['clam', 'alt', 'default', current_theme_to_set]
            theme_set_successfully = False
            for theme_name in preferred_themes:
                if theme_name in available_themes:
                    try: self.style.theme_use(theme_name); current_theme_to_set = theme_name; theme_set_successfully = True; log_message(self.log_area, f"  הוגדר בהצלחה Theme: {current_theme_to_set}"); break 
                    except tk.TclError: continue 
            if not theme_set_successfully: log_message(self.log_area, f"  לא הצלחתי להגדיר Theme מועדף. משתמש ב-Theme נוכחי/ברירת מחדל: {current_theme_to_set}.")
        except Exception as e: log_message(self.log_area, f"  שגיאה בהגדרת Theme: {e}. משתמש בברירת מחדל.")

    def check_aapt_availability_on_startup(self):
        def _check_aapt_in_thread():
            log_message(self.log_area, "\n--- בודק זמינות aapt באתחול (ברקע)... ---")
            aapt_path = get_aapt_path(self.log_area)
            if aapt_path: log_message(self.log_area, f"  בדיקת aapt (אתחול): aapt זוהה ב: {aapt_path}")
            else: log_message(self.log_area, "  בדיקת aapt (אתחול): aapt לא אותר. החילוץ הידני יהיה שיטת הגיבוי העיקרית.")
            log_message(self.log_area, "--- סיום בדיקת aapt באתחול ---")
        threading.Thread(target=_check_aapt_in_thread, daemon=True).start()
        
    def select_apks(self):
        files = filedialog.askopenfilenames(title="בחר קבצי APK", filetypes=(("APK files", "*.apk"), ("All files", "*.*")))
        if files: self.apk_files = list(files); count_text = f"{len(self.apk_files)} קבצים נבחרו"; self.lbl_apk_count_var.set(count_text); log_message(self.log_area, f"נבחרו {len(self.apk_files)} קבצי APK.")
        elif not self.apk_files: self.lbl_apk_count_var.set("לא נבחרו קבצים")
        self.update_extract_button_state()

    def select_output_dir(self):
        directory = filedialog.askdirectory(title="בחר תיקיית פלט")
        if directory: self.output_dir = directory; self.lbl_output_dir_var.set(f"תיקיית פלט: {os.path.basename(directory)}"); log_message(self.log_area, f"תיקיית הפלט נבחרה: {directory}")
        elif not self.output_dir: self.lbl_output_dir_var.set("לא נבחרה תיקייה")
        self.update_extract_button_state()

    def update_extract_button_state(self):
        if self.apk_files and self.output_dir: self.btn_extract.config(state=tk.NORMAL)
        else: self.btn_extract.config(state=tk.DISABLED)
            
    def extraction_progress_update(self, is_finished):
        if is_finished: self.update_extract_button_state(); messagebox.showinfo("סיום", "תהליך חילוץ האייקונים הסתיים. בדוק את הלוג לפרטים.", parent=self.root)

    def start_extraction_thread(self):
        if not self.apk_files: messagebox.showwarning("שגיאה", "אנא בחר לפחות קובץ APK אחד.", parent=self.root); return
        if not self.output_dir: messagebox.showwarning("שגיאה", "אנא בחר תיקיית פלט.", parent=self.root); return
        self.btn_extract.config(state=tk.DISABLED)
        log_message(self.log_area, "\n--- מתחיל חילוץ אייקונים (גרסה V4) ---")
        threading.Thread(target=process_apk_files, args=(self.apk_files, self.output_dir, self.log_area, lambda finished: self.root.after(0, self.extraction_progress_update, finished)), daemon=True).start()


if __name__ == "__main__":
    # ... (לוגיקת __main__ עם טיפול בשגיאות כמו קודם)
    crash_log_filename = "apk_extractor_crash_log.txt"
    try: main_root = tk.Tk(); app = App(main_root); main_root.mainloop()
    except Exception as e_main:
        error_type = type(e_main).__name__; error_message = str(e_main); full_traceback = traceback.format_exc()
        error_details_for_popup = f"שגיאה: {error_type}" + (f" - {error_message.splitlines()[0]}" if error_message else "")
        print(f"CRITICAL ERROR: {error_type} - {error_message}\n{full_traceback}", file=sys.stderr)
        try:
            with open(crash_log_filename, "w", encoding="utf-8") as f: f.write(f"Unhandled exception:\nType: {error_type}\nMessage: {error_message}\n\nTraceback:\n{full_traceback}")
            error_gui_root = tk.Tk(); error_gui_root.withdraw(); messagebox.showerror("שגיאה קריטית", f"אירעה שגיאה והתוכנית נסגרת.\nפרטים בקובץ: {os.path.abspath(crash_log_filename)}\n\n{error_details_for_popup}", parent=None); error_gui_root.destroy()
        except Exception as e_log_or_msgbox: print(f"Error handling critical error: {e_log_or_msgbox}", file=sys.stderr)
        sys.exit(1)